@@ -1,21 +1,19 @@
-mod util;
+mod cli;
+mod error;
 
 use ansi_term::Colour::{Green, White};
 use clap::StructOpt;
-use dialoguer::{theme::ColorfulTheme, Input, MultiSelect, Select};
+use cli::{Ferium, ProfileSubCommands, SubCommands};
+use dialoguer::{theme::ColorfulTheme, Confirm, Input, MultiSelect, Select};
+use error::{Error, Result};
 use ferinth::Ferinth;
 use furse::Furse;
+use libium::{config, launchermeta, misc};
 use octocrab::Octocrab;
-use std::{
+use std::path::PathBuf;
+use tokio::{
 	fs::{create_dir_all, OpenOptions},
-	io::Write,
-	path::PathBuf,
-};
-use util::{
-	cli::{Ferium, ProfileSubCommands, SubCommands},
-	ferium_error,
-	ferium_error::{FError, FResult},
-	json, launchermeta, wrappers,
+	io::{AsyncReadExt, AsyncWriteExt},
 };
 
 #[tokio::main]
@@ -28,21 +26,21 @@ async fn main() {
 	}
 }
 
-async fn actual_main() -> FResult<()> {
+async fn actual_main() -> Result<()> {
 	// Get the command to execute from Clap
-	// This also displays help page or version
+	// This also displays the help page or version
 	let cli_app = Ferium::parse();
 
 	// Check for an internet connection
 	if online::check(Some(1)).await.is_err() {
 		// If it takes more than 1 second
 		// show that we're checking the internet connection
-		// and wait for 4 more seconds
+		// and check for 4 more seconds
 		eprint!("Checking internet connection... ");
 		match online::check(Some(4)).await {
 			Ok(_) => println!("✓"),
 			Err(_) => {
-				return Err(FError::Quit(
+				return Err(Error::Quit(
 					"× Ferium requires an internet connection to work",
 				))
 			},
@@ -56,17 +54,21 @@ async fn actual_main() -> FResult<()> {
 		"A CurseForge API key is required to build. If you don't have one, you can bypass this by setting the variable to a blank string, however anything using the CurseForge API will not work."
 	));
 	// Ferium's config file
-	let config_file = json::get_config_file()?;
+	let mut config_file = config::get_config_file(config::config_file_path()).await?;
+	let mut config_file_contents = String::new();
+	config_file
+		.read_to_string(&mut config_file_contents)
+		.await?;
 	// Deserialise `config_file` to a config
-	let mut config: json::Config = match serde_json::from_reader(&config_file) {
+	let mut config: config::structs::Config = match serde_json::from_str(&config_file_contents) {
 		Ok(config) => config,
 		Err(err) => {
-			return Err(FError::QuitFormatted(format!(
+			return Err(Error::QuitFormatted(format!(
 				"Error decoding configuration file, {} at {:?} {}:{}",
 				// Error name
-				FError::JSONError(err.classify()),
+				Error::JSONError(err.classify()),
 				// File path so that users can find it
-				json::get_config_file_path(),
+				config::config_file_path(),
 				// Location within config file
 				err.line(),
 				err.column()
@@ -96,9 +98,8 @@ async fn actual_main() -> FResult<()> {
 		)
 		.await?;
 
-		// Update config file with new values
-		json::write_to_config(config_file, &config)?;
-
+		// Update config file with new values and quit
+		config::write_config(&mut config_file, &config).await?;
 		return Ok(());
 	}
 
@@ -107,30 +108,30 @@ async fn actual_main() -> FResult<()> {
 		profile
 	} else {
 		if config.profiles.is_empty() {
-			return Err(FError::Quit (
+			return Err(Error::Quit (
 				"There are no profiles configured. Add a profile to your config using `ferium profile create`"
 			));
 		}
 		// Default to first profile if index is set incorrectly
 		config.active_profile = 0;
-		json::write_to_config(config_file, &config)?;
-		return Err(FError::Quit(
+		config::write_config(&mut config_file, &config).await?;
+		return Err(Error::Quit(
 			"Active profile index points to a non existent profile. Switched to first profile",
 		));
 	};
 
 	// Run function(s) based on the sub(sub)command to be executed
 	match cli_app.subcommand {
-		SubCommands::AddModrinth { mod_id } => add_mod_modrinth(&modrinth, mod_id, profile).await?,
+		SubCommands::AddModrinth { project_id } => add_mod_modrinth(&modrinth, project_id, profile).await?,
 		SubCommands::AddGithub { owner, name } => {
 			add_repo_github(&github, owner, name, profile).await?;
 		},
 		SubCommands::AddCurseforge { project_id } => {
 			add_project_curseforge(&curseforge, project_id, profile).await?;
 		},
-		SubCommands::List { verbose } => {
+		SubCommands::List => {
 			check_empty_profile(profile)?;
-			list(&curseforge, &modrinth, &github, profile, verbose).await?;
+			list(&curseforge, &modrinth, &github, profile).await?;
 		},
 		SubCommands::Profile { subcommand } => match subcommand {
 			ProfileSubCommands::Configure {
@@ -154,7 +155,7 @@ async fn actual_main() -> FResult<()> {
 			no_patch_check,
 		} => {
 			check_empty_profile(profile)?;
-			create_dir_all(&profile.output_dir)?;
+			create_dir_all(&profile.output_dir).await?;
 			upgrade_curseforge(&curseforge, profile, no_patch_check).await?;
 			upgrade_modrinth(&modrinth, profile, no_patch_check).await?;
 			upgrade_github(&github, profile, no_picker).await?;
@@ -162,24 +163,24 @@ async fn actual_main() -> FResult<()> {
 	};
 
 	// Update config file with new values
-	json::write_to_config(config_file, &config)?;
+	config::write_config(&mut config_file, &config).await?;
 
 	Ok(())
 }
 
 /// Check if `profile` is empty, and if so return an error
-fn check_empty_profile(profile: &json::Profile) -> FResult<()> {
+fn check_empty_profile(profile: &config::structs::Profile) -> Result<()> {
 	if profile.github_repos.is_empty()
 		&& profile.modrinth_mods.is_empty()
 		&& profile.curse_projects.is_empty()
 	{
-		Err(FError::EmptyConfigFile)
+		Err(Error::EmptyConfigFile)
 	} else {
 		Ok(())
 	}
 }
 
-fn list_profiles(config: &json::Config) {
+fn list_profiles(config: &config::structs::Config) {
 	for profile in &config.profiles {
 		println!(
 			"{}
@@ -201,13 +202,13 @@ fn list_profiles(config: &json::Config) {
 }
 
 async fn create(
-	config: &mut json::Config,
+	config: &mut config::structs::Config,
 	game_version: Option<String>,
 	force_game_version: bool,
-	mod_loader: Option<json::ModLoaders>,
+	mod_loader: Option<config::structs::ModLoaders>,
 	name: Option<String>,
 	output_dir: Option<PathBuf>,
-) -> FResult<()> {
+) -> Result<()> {
 	match (game_version, mod_loader, name, output_dir) {
 		(Some(game_version), Some(mod_loader), Some(name), Some(output_dir)) => {
 			// If force game version is false
@@ -220,7 +221,7 @@ async fn create(
 					.any(|version| version.id == game_version)
 				{
 					// Then error out
-					return Err(FError::QuitFormatted(format!(
+					return Err(Error::QuitFormatted(format!(
 						"The game version {} does not exist",
 						game_version
 					)));
@@ -229,7 +230,7 @@ async fn create(
 			// Check that there isn't already a profile with the same name
 			for profile in &config.profiles {
 				if profile.name == name {
-					return Err(FError::QuitFormatted(format!(
+					return Err(Error::QuitFormatted(format!(
 						"A profile with name {} already exists",
 						name
 					)));
@@ -237,11 +238,11 @@ async fn create(
 			}
 			// Check that the output_dir isn't relative
 			if !output_dir.is_absolute() {
-				return Err(FError::Quit(
+				return Err(Error::Quit(
 					"The provided output directory is not absolute, i.e. it is a relative path",
 				));
 			}
-			config.profiles.push(json::Profile {
+			config.profiles.push(config::structs::Profile {
 				name,
 				output_dir,
 				game_version,
@@ -252,16 +253,73 @@ async fn create(
 			}); // Create profile
 		},
 		(None, None, None, None) => {
+			// Create profile using a UI
 			println!("Please enter the details for the new profile");
-			// Create profile using the UI
-			config
-				.profiles
-				.push(json::Profile::create_ui(config).await?);
+
+			// Let user pick mods directory
+			let mut selected_mods_dir = misc::get_mods_dir();
+			println!("The default mods directory is {:?}", selected_mods_dir);
+			if Confirm::with_theme(&ColorfulTheme::default())
+				.with_prompt("Would you like to specify a custom mods directory?")
+				.interact()?
+			{
+				if let Some(dir) = misc::pick_folder(&selected_mods_dir).await {
+					selected_mods_dir = dir;
+				};
+			}
+
+			let mut name = String::new();
+			let mut prompt = true;
+			while prompt {
+				name = Input::with_theme(&ColorfulTheme::default())
+					.with_prompt("What should this profile be called?")
+					.interact_text()?;
+
+				prompt = false;
+				for profile in &config.profiles {
+					if profile.name == name {
+						println!("A profile with name {} already exists!", name);
+						prompt = true;
+					}
+				}
+			}
+
+			// Let user pick Minecraft version
+			let mut latest_versions: Vec<String> =
+				misc::get_latest_mc_versions(10, launchermeta::get_version_manifest().await?)?;
+			println!();
+			let selected_version = Select::with_theme(&ColorfulTheme::default())
+				.with_prompt("Which version of Minecraft do you play?")
+				.items(&latest_versions)
+				.default(0)
+				.interact()?;
+			let selected_version = latest_versions.swap_remove(selected_version);
+
+			// Let user pick mod loader
+			let mod_loaders = ["Fabric", "Forge"];
+			let selected_loader = if Select::with_theme(&ColorfulTheme::default())
+				.with_prompt("Which mod loader do you use?")
+				.items(&mod_loaders)
+				.interact()? == 0
+			{
+				config::structs::ModLoaders::Fabric
+			} else {
+				config::structs::ModLoaders::Forge
+			};
+			config.profiles.push(config::structs::Profile {
+				name,
+				output_dir: selected_mods_dir,
+				curse_projects: Vec::new(),
+				modrinth_mods: Vec::new(),
+				github_repos: Vec::new(),
+				game_version: selected_version,
+				mod_loader: selected_loader,
+			});
 		},
 		// Either all or none of these options should exist
 		// TODO: make this into a group in the Clap app
 		_ => {
-			return Err(FError::Quit(
+			return Err(Error::Quit(
 				"Provide all four arguments to create a profile using options",
 			))
 		},
@@ -271,7 +329,7 @@ async fn create(
 	Ok(())
 }
 
-fn delete(config: &mut json::Config, profile_name: Option<String>) -> FResult<()> {
+fn delete(config: &mut config::structs::Config, profile_name: Option<String>) -> Result<()> {
 	let selection = match profile_name {
 		// If the profile name has been provided as an option
 		Some(profile_name) => {
@@ -287,7 +345,7 @@ fn delete(config: &mut json::Config, profile_name: Option<String>) -> FResult<()
 				// If the profile is found, return its index
 				Ok(selection) => selection,
 				// Else return an error
-				Err(_) => return Err(FError::Quit("The profile name provided does not exist")),
+				Err(_) => return Err(Error::Quit("The profile name provided does not exist")),
 			}
 		},
 		None => {
@@ -320,24 +378,20 @@ fn delete(config: &mut json::Config, profile_name: Option<String>) -> FResult<()
 	Ok(())
 }
 
-fn switch(config: &mut json::Config, profile_name: Option<String>) -> FResult<()> {
+fn switch(config: &mut config::structs::Config, profile_name: Option<String>) -> Result<()> {
 	if config.profiles.len() < 2 {
-		Err(FError::Quit("There is less than 2 profiles in your config"))
+		Err(Error::Quit("There is less than 2 profiles in your config"))
 	} else if let Some(profile_name) = profile_name {
-		// Sort profiles by name
-		config
-			.profiles
-			.sort_unstable_by_key(|profile| profile.name.clone());
-		// Binary search the profile
 		match config
 			.profiles
-			.binary_search_by_key(&&profile_name, |profile| &profile.name)
+			.iter()
+			.position(|profile| profile.name == profile_name)
 		{
-			Ok(selection) => {
+			Some(selection) => {
 				config.active_profile = selection;
 				Ok(())
 			},
-			Err(_) => Err(FError::Quit("The profile provided does not exist")),
+			None => Err(Error::Quit("The profile provided does not exist")),
 		}
 	} else {
 		let profile_names = config
@@ -357,12 +411,12 @@ fn switch(config: &mut json::Config, profile_name: Option<String>) -> FResult<()
 }
 
 async fn configure(
-	profile: &mut json::Profile,
+	profile: &mut config::structs::Profile,
 	game_version: Option<String>,
-	mod_loader: Option<json::ModLoaders>,
+	mod_loader: Option<config::structs::ModLoaders>,
 	name: Option<String>,
 	output_dir: Option<PathBuf>,
-) -> FResult<()> {
+) -> Result<()> {
 	let mut interactive = true;
 
 	if let Some(game_version) = game_version {
@@ -411,17 +465,20 @@ async fn configure(
 							White.bold().paint("Pick a mod output directory   "),
 						);
 						// Let user pick output directory
-						if let Some(dir) = wrappers::pick_folder(&profile.output_dir).await {
+						if let Some(dir) = misc::pick_folder(&profile.output_dir).await {
 							profile.output_dir = dir;
 						}
 						println!(
 							"{}",
-							Green.paint(profile.output_dir.to_str().ok_or(FError::OptionError)?)
+							Green.paint(profile.output_dir.to_str().ok_or(Error::OptionError)?)
 						);
 					},
 					1 => {
 						// Let user pick mc version from latest 10 versions
-						let mut versions = wrappers::get_latest_mc_versions(10).await?;
+						let mut versions = misc::get_latest_mc_versions(
+							10,
+							launchermeta::get_version_manifest().await?,
+						)?;
 						let index = Select::with_theme(&ColorfulTheme::default())
 							.with_prompt("Select a Minecraft version")
 							.items(&versions)
@@ -438,14 +495,14 @@ async fn configure(
 							.with_prompt("Pick a mod loader")
 							.items(&mod_loaders)
 							.default(match profile.mod_loader {
-								json::ModLoaders::Fabric => 0,
-								json::ModLoaders::Forge => 1,
+								config::structs::ModLoaders::Fabric => 0,
+								config::structs::ModLoaders::Forge => 1,
 							})
 							.interact_opt()?;
 						if index == Some(0) {
-							profile.mod_loader = json::ModLoaders::Fabric;
+							profile.mod_loader = config::structs::ModLoaders::Fabric;
 						} else if index == Some(1) {
-							profile.mod_loader = json::ModLoaders::Forge;
+							profile.mod_loader = config::structs::ModLoaders::Forge;
 						}
 					},
 					3 => {
@@ -473,16 +530,16 @@ async fn remove(
 	curseforge: &Furse,
 	modrinth: &Ferinth,
 	github: &Octocrab,
-	profile: &mut json::Profile,
+	profile: &mut config::structs::Profile,
 	mod_names: Option<Vec<String>>,
-) -> FResult<()> {
+) -> Result<()> {
 	let mut names: Vec<String> = Vec::new();
 
 	// Get the names of the mods
 	eprint!("Gathering mod names... ");
-	for mod_id in &profile.modrinth_mods {
-		let mod_ = modrinth.get_mod(mod_id).await?;
-		names.push(mod_.title);
+	for project_id in &profile.modrinth_mods {
+		let project = modrinth.get_project(project_id).await?;
+		names.push(project.title);
 	}
 
 	for repo_name in &profile.github_repos {
@@ -518,7 +575,7 @@ async fn remove(
 
 				// If a mod is not found, throw an error
 				if !found_mod {
-					return Err(FError::QuitFormatted(format!(
+					return Err(Error::QuitFormatted(format!(
 						"A mod called {} is not present in this profile",
 						mod_name
 					)));
@@ -544,7 +601,7 @@ async fn remove(
 
 	// For each mod to remove
 	for index in items_to_remove {
-		// If index is larger than the length of the repos mod_ids, then the index is for curse projects
+		// If index is larger than the length of the repos and modrinth project_ids, then the index is for curse projects
 		if index >= (profile.modrinth_mods.len() + profile.github_repos.len()) {
 			// Offset the index by the proper amount
 			let index = index - (profile.modrinth_mods.len() + profile.github_repos.len());
@@ -559,7 +616,7 @@ async fn remove(
 
 			// Remove item from profile's repos
 			profile.github_repos.swap_remove(index);
-		// Or else its for the mod_ids
+		// Or else its for the modrinth project_ids
 		} else {
 			// Remove item from profile' mod ids
 			profile.modrinth_mods.swap_remove(index);
@@ -574,8 +631,8 @@ async fn add_repo_github(
 	github: &Octocrab,
 	owner: String,
 	repo_name: String,
-	profile: &mut json::Profile,
-) -> FResult<()> {
+	profile: &mut config::structs::Profile,
+) -> Result<()> {
 	eprint!("Adding GitHub repository... ");
 
 	// Get repository and releases data
@@ -585,14 +642,14 @@ async fn add_repo_github(
 	let repo_name_split = repo
 		.full_name
 		.as_ref()
-		.ok_or(FError::OptionError)?
+		.ok_or(Error::OptionError)?
 		.split('/')
 		.collect::<Vec<_>>();
 	let repo_name = (repo_name_split[0].into(), repo_name_split[1].into());
 
 	// Check if repo has already been added
 	if profile.github_repos.contains(&repo_name) {
-		return Err(FError::Quit("× Repository already added to profile!"));
+		return Err(Error::Quit("× Repository already added to profile!"));
 	}
 
 	let releases = repo_handler.releases().list().send().await?;
@@ -614,7 +671,7 @@ async fn add_repo_github(
 		profile.github_repos.push(repo_name);
 		println!("✓");
 	} else {
-		return Err(FError::Quit("× Repository does not release mods!"));
+		return Err(Error::Quit("× Repository does not release mods!"));
 	}
 
 	Ok(())
@@ -623,29 +680,29 @@ async fn add_repo_github(
 /// Check if mod with ID `mod_id` exists, if so add that mod to `profile`
 async fn add_mod_modrinth(
 	modrinth: &Ferinth,
-	mod_id: String,
-	profile: &mut json::Profile,
-) -> FResult<()> {
+	project_id: String,
+	profile: &mut config::structs::Profile,
+) -> Result<()> {
 	eprint!("Adding Modrinth mod... ");
 
 	// Check if mod exists
-	match modrinth.get_mod(&mod_id).await {
-		Ok(mod_) => {
+	match modrinth.get_project(&project_id).await {
+		Ok(project) => {
 			// Check if mod has already been added
-			if profile.modrinth_mods.contains(&mod_.id) {
-				return Err(FError::Quit("× Mod already added to profile!"));
+			if profile.modrinth_mods.contains(&project.id) {
+				return Err(Error::Quit("× Mod already added to profile!"));
 			}
 			// And if it hasn't, append mod to profile and write
-			profile.modrinth_mods.push(mod_.id);
-			println!("✓ ({})", mod_.title);
+			profile.modrinth_mods.push(project.id);
+			println!("✓ ({})", project.title);
 
 			Ok(())
 		},
 		Err(_) => {
 			// Else return an error
-			Err(FError::QuitFormatted(format!(
+			Err(Error::QuitFormatted(format!(
 				"× Mod with ID `{}` does not exist!",
-				mod_id
+				project_id
 			)))
 		},
 	}
@@ -654,22 +711,22 @@ async fn add_mod_modrinth(
 async fn add_project_curseforge(
 	curseforge: &Furse,
 	project_id: i32,
-	profile: &mut json::Profile,
-) -> FResult<()> {
+	profile: &mut config::structs::Profile,
+) -> Result<()> {
 	eprint!("Adding CurseForge mod... ");
 
 	// Check if project exists
 	match curseforge.get_mod(project_id).await {
 		Ok(project) => {
 			if profile.curse_projects.contains(&project.id) {
-				Err(FError::Quit("× Project already added to profile!"))
+				Err(Error::Quit("× Project already added to profile!"))
 			} else {
 				profile.curse_projects.push(project.id);
 				println!("✓ ({})", project.name);
 				Ok(())
 			}
 		},
-		Err(err) => Err(FError::QuitFormatted(format!(
+		Err(err) => Err(Error::QuitFormatted(format!(
 			"× Project with ID `{}` does not exist! ({})",
 			project_id, err
 		))),
@@ -681,9 +738,8 @@ async fn list(
 	curseforge: &Furse,
 	modrinth: &Ferinth,
 	github: &Octocrab,
-	profile: &json::Profile,
-	verbose: bool,
-) -> FResult<()> {
+	profile: &config::structs::Profile,
+) -> Result<()> {
 	for project_id in &profile.curse_projects {
 		let project = curseforge.get_mod(*project_id).await?;
 		let mut authors = String::new();
@@ -724,121 +780,86 @@ async fn list(
 		);
 	}
 
-	for mod_id in &profile.modrinth_mods {
-		// Get mod metadata
-		let mod_ = modrinth.get_mod(mod_id).await?;
-		if verbose {
-			let team_members = modrinth.list_team_members(&mod_.team).await?;
+	for project_id in &profile.modrinth_mods {
+		// Get project metadata
+		let project = modrinth.get_project(project_id).await?;
+		let team_members = modrinth.list_team_members(&project.team).await?;
 
-			// Get the usernames of all the developers
-			let mut developers = String::new();
-			for member in team_members {
-				let user = modrinth.get_user(&member.user_id).await?;
-				developers.push_str(&user.username);
-				developers.push_str(", ");
-			}
-			// Trim trailing ', '
-			developers.truncate(developers.len() - 2);
+		// Get the usernames of all the developers
+		let mut developers = String::new();
+		for member in team_members {
+			developers.push_str(&member.user.username);
+			developers.push_str(", ");
+		}
+		// Trim trailing ', '
+		developers.truncate(developers.len() - 2);
 
-			println!(
-				"{}
+		println!(
+			"{}
             \r  {}\n
             \r  Link:           https://modrinth.com/mod/{}
             \r  Source:         Modrinth Mod
             \r  Open Source:    {}
             \r  Downloads:      {}
             \r  Developers:     {}
-            \r  Client side:    {}
-            \r  Server side:    {}
+            \r  Client side:    {:?}
+            \r  Server side:    {:?}
             \r  License:        {}{}\n",
-				mod_.title,
-				mod_.description,
-				mod_.slug,
-				mod_.source_url
-					.map_or("No".into(), |url| { format!("Yes ({})", url) }),
-				mod_.downloads,
-				developers,
-				mod_.client_side,
-				mod_.server_side,
-				mod_.license.name,
-				mod_.license
-					.url
-					.map_or("".into(), |url| { format!(" ({})", url) }),
-			);
-		} else {
-			println!(
-				"{}
-                \r  {}\n
-                \r  Link:     https://modrinth.com/mod/{}
-                \r  Source:   Modrinth Mod
-                \r  Code:     {}
-                \r  License:  {}\n",
-				mod_.title,
-				mod_.description,
-				mod_.slug,
-				mod_.source_url
-					.map_or("Closed source".into(), |url| { url }),
-				mod_.license.name
-			);
-		}
+			project.title,
+			project.description,
+			project.slug,
+			project.source_url
+				.map_or("No".into(), |url| { format!("Yes ({})", url) }),
+			project.downloads,
+			developers,
+			project.client_side,
+			project.server_side,
+			project.license.name,
+			project.license
+				.url
+				.map_or("".into(), |url| { format!(" ({})", url) }),
+		);
 	}
 
 	for repo_name in &profile.github_repos {
 		// Get repository metadata
 		let repo_handler = github.repos(&repo_name.0, &repo_name.1);
 		let repo = repo_handler.get().await?;
-		if verbose {
-			let releases = repo_handler.releases().list().send().await?;
-			let mut downloads = 0;
+		let releases = repo_handler.releases().list().send().await?;
+		let mut downloads = 0;
 
-			// Calculate number of downloads
-			for release in releases {
-				for asset in release.assets {
-					downloads += asset.download_count;
-				}
+		// Calculate number of downloads
+		for release in releases {
+			for asset in release.assets {
+				downloads += asset.download_count;
 			}
+		}
 
-			// Print repository data formatted
-			println!(
-				"{}{}\n
+		// Print repository data formatted
+		println!(
+			"{}{}\n
             \r  Link:           {}
             \r  Source:         GitHub Repository
             \r  Downloads:      {}
             \r  Developer:      {}{}\n",
-				repo.name,
-				repo.description
-					.map_or("".into(), |description| { format!("\n  {}", description) }),
-				repo.html_url.ok_or(FError::OptionError)?,
-				downloads,
-				repo.owner.ok_or(FError::OptionError)?.login,
-				if let Some(license) = repo.license {
-					format!(
-						"\n  License:        {}{}",
-						license.name,
-						license
-							.html_url
-							.map_or("".into(), |url| { format!(" ({})", url) })
-					)
-				} else {
-					"".into()
-				},
-			);
-		} else {
-			println!(
-				"{}{}\n
-                \r  Link:     {}
-                \r  Source:   GitHub Repository{}\n",
-				repo.name,
-				repo.description
-					.map_or("".into(), |description| { format!("\n  {}", description) }),
-				repo.html_url.ok_or(FError::OptionError)?,
-				if let Some(license) = repo.license {
-					format!("\n  License:  {}", license.name)
-				} else {
-					"".into()
-				},
-			);
-		}
+			repo.name,
+			repo.description
+				.map_or("".into(), |description| { format!("\n  {}", description) }),
+			repo.html_url.ok_or(Error::OptionError)?,
+			downloads,
+			repo.owner.ok_or(Error::OptionError)?.login,
+			if let Some(license) = repo.license {
+				format!(
+					"\n  License:        {}{}",
+					license.name,
+					license
+						.html_url
+						.map_or("".into(), |url| { format!(" ({})", url) })
+				)
+			} else {
+				"".into()
+			},
+		);
 	}
 
 	Ok(())
@@ -846,9 +867,9 @@ async fn list(
 
 async fn upgrade_curseforge(
 	curseforge: &Furse,
-	profile: &json::Profile,
+	profile: &config::structs::Profile,
 	no_patch_check: bool,
-) -> FResult<()> {
+) -> Result<()> {
 	for project_id in &profile.curse_projects {
 		let project = curseforge.get_mod(*project_id).await?;
 		println!("Downloading {}", project.name);
@@ -862,7 +883,7 @@ async fn upgrade_curseforge(
 		files.reverse();
 
 		let mut latest_compatible_file = None;
-		let game_version_to_check = wrappers::remove_semver_patch(&profile.game_version)?;
+		let game_version_to_check = misc::remove_semver_patch(&profile.game_version)?;
 
 		for file in &files {
 			if no_patch_check {
@@ -901,17 +922,18 @@ async fn upgrade_curseforge(
 					.write(true)
 					.truncate(true)
 					.create(true)
-					.open(profile.output_dir.join(&latest_compatible_file.file_name))?;
+					.open(profile.output_dir.join(&latest_compatible_file.file_name))
+					.await?;
 
 				let file_contents = curseforge
 					.download_mod_file_from_file(latest_compatible_file)
 					.await?;
 
-				mod_file.write_all(&file_contents)?;
+				mod_file.write_all(&file_contents).await?;
 				println!("✓\n");
 			},
 			None => {
-				return Err(FError::QuitFormatted(format!(
+				return Err(Error::QuitFormatted(format!(
 					"× No version of {} is compatible for {} {}",
 					project.name, profile.mod_loader, profile.game_version,
 				)));
@@ -925,16 +947,16 @@ async fn upgrade_curseforge(
 /// Download and install all the GitHub mods in `profile`
 async fn upgrade_github(
 	github: &Octocrab,
-	profile: &json::Profile,
+	profile: &config::structs::Profile,
 	no_picker: bool,
-) -> FResult<()> {
+) -> Result<()> {
 	for repo_name in &profile.github_repos {
 		println!("Downloading {}", repo_name.1);
 		eprint!("  [1] Getting release information... ");
 
 		let repo_handler = github.repos(&repo_name.0, &repo_name.1);
 		let releases = repo_handler.releases().list().send().await?;
-		let version_to_check = wrappers::remove_semver_patch(&profile.game_version)?;
+		let version_to_check = misc::remove_semver_patch(&profile.game_version)?;
 
 		// A vector of assets that are compatible with the game version and mod loader
 		let mut asset_candidates = Vec::new();
@@ -958,11 +980,11 @@ async fn upgrade_github(
                     // Check if the game version is compatible
                     && (
                         // Check the release body
-                        release.body.as_ref().ok_or(FError::OptionError)?.contains(&version_to_check)
+                        release.body.as_ref().ok_or(Error::OptionError)?.contains(&version_to_check)
                         // the asset's name
                         || asset.name.contains(&version_to_check)
 						// and even the release name
-                        || release.name.as_ref().ok_or(FError::OptionError)?.contains(&version_to_check))
+                        || release.name.as_ref().ok_or(Error::OptionError)?.contains(&version_to_check))
                     // Check if its a JAR file
                     && asset.name.contains("jar")
 				{
@@ -978,7 +1000,7 @@ async fn upgrade_github(
 			asset_candidates[0]
 		// If none were found, error out
 		} else if asset_candidates.is_empty() {
-			return Err(FError::Quit(
+			return Err(Error::Quit(
 				"× Could not find a compatible asset to download",
 			));
 		// If more than 1 was found, let the user select which one to use
@@ -1013,10 +1035,11 @@ async fn upgrade_github(
 			.write(true)
 			.truncate(true)
 			.create(true)
-			.open(profile.output_dir.join(&asset_to_download.name))?;
+			.open(profile.output_dir.join(&asset_to_download.name))
+			.await?;
 
 		// Write download to mod JAR file
-		mod_file.write_all(&contents)?;
+		mod_file.write_all(&contents).await?;
 		println!("✓\n");
 	}
 
@@ -1026,18 +1049,18 @@ async fn upgrade_github(
 /// Download and install all Modrinth mods in `profile`
 async fn upgrade_modrinth(
 	modrinth: &Ferinth,
-	profile: &json::Profile,
+	profile: &config::structs::Profile,
 	no_patch_check: bool,
-) -> FResult<()> {
-	for mod_id in &profile.modrinth_mods {
+) -> Result<()> {
+	for project_id in &profile.modrinth_mods {
 		// Get mod metadata
-		let mod_ = modrinth.get_mod(mod_id).await?;
-		println!("Downloading {}", mod_.title);
+		let project = modrinth.get_project(project_id).await?;
+		println!("Downloading {}", project.title);
 
 		eprint!("  [1] Getting version information... ");
 		// Get the versions of the mod
-		let versions = modrinth.list_versions(&mod_.id).await?;
-		let game_version_to_check = wrappers::remove_semver_patch(&profile.game_version)?;
+		let versions = modrinth.list_versions(&project.id).await?;
+		let game_version_to_check = misc::remove_semver_patch(&profile.game_version)?;
 
 		let mut latest_compatible_version = None;
 
@@ -1073,9 +1096,9 @@ async fn upgrade_modrinth(
 			Some(version) => version,
 			// If version compatible with game version does not exist, throw an error
 			None => {
-				return Err(FError::QuitFormatted(format!(
+				return Err(Error::QuitFormatted(format!(
 					"× No version of {} is compatible for {} {}",
-					mod_.title, profile.mod_loader, profile.game_version,
+					project.title, profile.mod_loader, profile.game_version,
 				)));
 			},
 		};
@@ -1095,10 +1118,11 @@ async fn upgrade_modrinth(
 			.write(true)
 			.truncate(true)
 			.create(true)
-			.open(profile.output_dir.join(&latest_version.files[0].filename))?;
+			.open(profile.output_dir.join(&latest_version.files[0].filename))
+			.await?;
 
 		// Write contents to JAR file
-		mod_file.write_all(&contents)?;
+		mod_file.write_all(&contents).await?;
 		println!("✓\n");
 	}
 
